from distutils.core import setup

setup(
    name='pydataframe',
    version='0.1.1',
    packages=['pydataframe',],
    license='BSD',
    url='http://code.google.com/p/pydataframe/',
    author='Florian Finkernagel',
    author_email='finkernagel@coonabibba.de',
    long_description=open('README.txt').read(),
)
